@@ -1,6 +1,11 @@
+// SPDX-FileCopyrightText: 2022 Uniontech Software Technology Co.,Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
 #include "dwaylandshellmanager.h"
 #include "dkeyboard.h"
 #include "global.h"
+#include "../xcb/utility.h"
 
 #define protected public
 #include <qwindow.h>
@@ -8,6 +13,7 @@
 
 #include <QtWaylandClientVersion>
 #include <QLoggingCategory>
+#include <QPainterPath>
 
 #ifndef QT_DEBUG
 Q_LOGGING_CATEGORY(dwlp, "dtk.wayland.plugin" , QtInfoMsg);
@@ -21,6 +27,8 @@ DPP_USE_NAMESPACE
 #define CHECK_PREFIX(key) (key.startsWith(_DWAYALND_) || key.startsWith("_d_"))
 #define wlDisplay reinterpret_cast<wl_display *>(QGuiApplication::platformNativeInterface()->nativeResourceForWindow("display", nullptr))
 
+Q_DECLARE_METATYPE(QPainterPath)
+
 namespace QtWaylandClient {
 
 namespace {
@@ -37,6 +45,10 @@ namespace {
     DDEPointer *kwayland_dde_pointer = nullptr;
     FakeInput *kwayland_dde_fake_input = nullptr;
     DDEKeyboard *kwayland_dde_keyboard = nullptr;
+    BlurManager *kwayland_blur_manager = nullptr;
+    Surface *kwayland_surface = nullptr;
+    Compositor *kwayland_compositor = nullptr;
+    PlasmaWindowManagement *kwayland_manage = nullptr;
 };
 
 QList<QPointer<QWaylandWindow>> DWaylandShellManager::send_property_window_list;
@@ -89,6 +101,53 @@ static DDEShellSurface *ensureDDEShellSurface(QWaylandShellSurface *self)
     return createDDESurface(self->window());
 }
 
+static Surface *ensureSurface(QWaylandWindow *wlWindow)
+{
+    if (!kwayland_surface) {
+        qCWarning(dwlp) << "invalid wayland surface";
+        return nullptr;
+    }
+    if (!wlWindow->window()) {
+        qCWarning(dwlp) << "invalid wlWindow";
+        return nullptr;
+    }
+    return kwayland_surface->fromWindow(wlWindow->window());
+}
+
+static Blur *ensureBlur(Surface *surface, QObject *parent = nullptr)
+{
+    if (parent) {
+        if (auto *blur = parent->findChild<Blur *>(QString(), Qt::FindDirectChildrenOnly))
+            return blur;
+    }
+    if (!kwayland_blur_manager) {
+        qCWarning(dwlp) << "invalid blur manager";
+        return nullptr;
+    }
+    return kwayland_blur_manager->createBlur(surface, parent);
+}
+
+static Region *ensureRegion(QObject *parent = nullptr)
+{
+    if (parent) {
+        if (auto *region = parent->findChild<Region *>(QString(), Qt::FindDirectChildrenOnly)) {
+            return region;
+        }
+    }
+    if (!kwayland_compositor) {
+        qCWarning(dwlp) << "invalid wayland compositor";
+        return nullptr;
+    }
+    return kwayland_compositor->createRegion(parent);
+}
+
+static void checkIsDWayland(const QString &function)
+{
+#ifndef D_DEEPIN_IS_DWAYLAND
+    qCWarning(dwlp) << "This package is not compiled as dwayland, and [" << function << "] not support in this version.";
+#endif
+}
+
 DWaylandShellManager::DWaylandShellManager()
     : m_registry (new Registry())
 {
@@ -144,24 +203,43 @@ void DWaylandShellManager::sendProperty(QWaylandShellSurface *self, const QStrin
         }
         if (!name.compare(splitWindowOnScreen)) {
             using KWayland::Client::DDEShellSurface;
-            bool ok = false;
-            qreal leftOrRight  = value.toInt(&ok);
-            if (ok) {
-                dde_shell_surface->requestSplitWindow(DDEShellSurface::SplitType(leftOrRight));
-                qCDebug(dwlp) << "requestSplitWindow value: " << leftOrRight;
+            const auto tmp = value.toList();
+            if (tmp.size() >= 2) {
+                const auto &position = tmp[0].toInt();
+                const auto &type = tmp[1].toInt();
+                checkIsDWayland("requestSplitWindow()");
+#ifdef D_DEEPIN_IS_DWAYLAND
+                dde_shell_surface->requestSplitWindow(DDEShellSurface::SplitType(position), DDEShellSurface::SplitMode(type));
+                qCDebug(dwlp) << "requestSplitWindow splitType: " << position << " mode: " << type;
+#endif
             } else {
                 qCWarning(dwlp) << "invalid property: " << name << value;
             }
             wlWindow->window()->setProperty(splitWindowOnScreen, 0);
         }
         if (!name.compare(supportForSplittingWindow)) {
-            wlWindow->window()->setProperty(supportForSplittingWindow, dde_shell_surface->isSplitable());
+            checkIsDWayland("getSplitable()");
+#ifdef D_DEEPIN_IS_DWAYLAND
+            wlWindow->window()->setProperty(supportForSplittingWindow, dde_shell_surface->getSplitable());
+#endif
             return;
         }
         if (!name.compare(windowInWorkSpace)) {
             dde_shell_surface->requestOnAllDesktops(value.toBool());
             qCDebug(dwlp()) << "### requestOnAllDesktops" << name << value;
         }
+        if (!name.compare(enableBlurWindow)) {
+            qCDebug(dwlp) << "### enableBlurWindow" << name << value;
+            if (!value.isValid()) {
+                qCWarning(dwlp) << "invalid enableBlurWindow";
+                return;
+            }
+            setEnableBlurWidow(wlWindow, value);
+        }
+        if (!name.compare(windowBlurAreas) || !name.compare(windowBlurPaths)) {
+            qCDebug(dwlp) << "### requestWindowBlur" << name << value;
+            updateWindowBlurAreasForWM(wlWindow, name, value);
+        }
     }
 
     // 将popup的窗口设置为tooltop层级, 包括qmenu，combobox弹出窗口
@@ -196,6 +274,11 @@ void DWaylandShellManager::sendProperty(QWaylandShellSurface *self, const QStrin
     if (QStringLiteral(_DWAYALND_ "window-position") == name) {
         ksurface->setPosition(value.toPoint());
     }
+
+    if (QStringLiteral(_DWAYALND_ "dock-appitem-geometry") == name) {
+        setDockAppItemMinimizedGeometry(self, value);
+    }
+
     static const QMap<PlasmaShellSurface::Role, QStringList> role2type = {
         {PlasmaShellSurface::Role::Normal, {"normal"}},
         {PlasmaShellSurface::Role::Desktop, {"desktop"}},
@@ -456,6 +539,42 @@ void DWaylandShellManager::createDDEPointer()
     });
 }
 
+void DWaylandShellManager::createBlurManager(quint32 name, quint32 version)
+{
+    kwayland_blur_manager = registry()->createBlurManager(name, version);
+    if (!kwayland_blur_manager) {
+        qCWarning(dwlp) << "kwayland_blur_manager create failed.";
+        return;
+    }
+}
+
+void DWaylandShellManager::createCompositor(quint32 name, quint32 version)
+{
+    kwayland_compositor = registry()->createCompositor(name, version);
+    if (!kwayland_compositor) {
+        qCWarning(dwlp) << "kwayland_compositor create failed.";
+        return;
+    }
+}
+
+void DWaylandShellManager::createSurface()
+{
+    if (!kwayland_compositor) {
+        qCWarning(dwlp) << "kwayland_compositor is invalid.";
+        return;
+    }
+    kwayland_surface = kwayland_compositor->createSurface();
+    if (!kwayland_surface) {
+        qCWarning(dwlp) << "kwayland_surface create failed.";
+        return;
+    }
+}
+
+void DWaylandShellManager::createPlasmaWindowManagement(KWayland::Client::Registry *registry, quint32 name, quint32 version)
+{
+    kwayland_manage = registry->createPlasmaWindowManagement(name, version, registry->parent());
+}
+
 void DWaylandShellManager::requestActivateWindow(QPlatformWindow *self)
 {
     HookCall(self, &QPlatformWindow::requestActivateWindow);
@@ -623,6 +742,45 @@ void DWaylandShellManager::setDockStrut(QWaylandShellSurface *surface, const QVa
     kwayland_strut->setStrutPartial(getWindowWLSurface(surface->window()), dockStrut);
 }
 
+/*
+ * @brief setDockAppItemMinimizedGeometry 设置驻留任务栏应用窗口最小化的位置信息
+ * @param surface
+ * @param var[0]:应用 窗口Id
+ *        var[1]:app window x
+ *        var[2]:app window y
+ *        var[3]:app window width
+ *        var[4]:app window height
+ */
+void DWaylandShellManager::setDockAppItemMinimizedGeometry(QWaylandShellSurface *surface, const QVariant var)
+{
+    if (!surface) {
+        return;
+    }
+
+    for (auto w : kwayland_manage->windows()){
+        if (w->windowId() == var.toList()[0].toUInt()) {
+            auto x = var.toList()[1].toInt();
+            auto y = var.toList()[2].toInt();
+            auto width = var.toList()[3].toInt();
+            auto height = var.toList()[4].toInt();
+
+            QRect r(QPoint(x, y), QSize(width, height));
+
+            if (w) {
+                auto s = ensureSurface(surface->window());
+                if (!s) {
+                    qCWarning(dwlp) << "invalid surface";
+                    return;
+                }
+
+                w->setMinimizedGeometry(s, r);
+            }
+
+            return;
+        }
+    }
+}
+
 /*
  * @brief setCursorPoint  设置光标在屏幕中的绝对位置
  * @param pos
@@ -639,6 +797,100 @@ void DWaylandShellManager::setCursorPoint(QPointF pos) {
     kwayland_dde_fake_input->requestPointerMoveAbsolute(pos);
 }
 
+void DWaylandShellManager::setEnableBlurWidow(QWaylandWindow *wlWindow, const QVariant &value)
+{
+    auto surface = ensureSurface(wlWindow);
+    if (value.toBool()) {
+        auto blur = ensureBlur(surface, surface);
+        if (!blur) {
+            qCWarning(dwlp) << "invalid blur";
+            return;
+        }
+        auto region = ensureRegion(surface);
+        if (!region) {
+            qCWarning(dwlp) << "invalid region";
+            return;
+        }
+        blur->setRegion(region);
+        blur->commit();
+        if (!kwayland_surface) {
+            qCWarning(dwlp) << "invalid kwayland_surface";
+            return;
+        }
+        kwayland_surface->commit(Surface::CommitFlag::None);
+    } else {
+        if (!kwayland_blur_manager) {
+            qCWarning(dwlp) << "invalid kwayland_blur_manager";
+            return;
+        }
+        kwayland_blur_manager->removeBlur(surface);
+        if (!kwayland_surface) {
+            qCWarning(dwlp) << "invalid kwayland_surface";
+            return;
+        }
+        kwayland_surface->commit(Surface::CommitFlag::None);
+        // 取消模糊效果的更新需要主动调用应用侧的窗口
+        if (QWidgetWindow *widgetWin = static_cast<QWidgetWindow*>(wlWindow->window())) {
+            if (auto widget = widgetWin->widget()) {
+                widget->update();
+            }
+        }
+    }
+}
+
+void DWaylandShellManager::updateWindowBlurAreasForWM(QWaylandWindow *wlWindow, const QString &name, const QVariant &value)
+{
+    if (!wlWindow->waylandScreen()) {
+        return;
+    }
+    auto screen = wlWindow->waylandScreen()->screen();
+    if (!screen) {
+        return;
+    }
+    auto devicePixelRatio = screen->devicePixelRatio();
+
+    auto surface = ensureSurface(wlWindow);
+    if (!surface) {
+        qCWarning(dwlp) << "invalid surface";
+        return;
+    }
+    auto blur = ensureBlur(surface, surface);
+    if (!blur) {
+        qCWarning(dwlp) << "invalid blur";
+        return;
+    }
+    auto region = kwayland_compositor->createRegion(surface);
+
+    if (!name.compare(windowBlurAreas)) {
+        const QVector<quint32> &tmpV = qvariant_cast<QVector<quint32>>(value);
+        const QVector<Utility::BlurArea> &blurAreas = *(reinterpret_cast<const QVector<Utility::BlurArea>*>(&tmpV));
+        if (blurAreas.isEmpty()) {
+            qCWarning(dwlp) << "invalid BlurAreas";
+            return;
+        }
+        for (auto ba : blurAreas) {
+            ba *= devicePixelRatio;
+            QPainterPath path;
+            path.addRoundedRect(ba.x, ba.y, ba.width, ba.height, ba.xRadius, ba.yRaduis);
+            region->add(path.toFillPolygon().toPolygon());
+        }
+    } else {
+        const QList<QPainterPath> paths = qvariant_cast<QList<QPainterPath>>(value);
+        if (paths.isEmpty()) {
+            qCWarning(dwlp) << "invalid BlurPaths";
+            return;
+        }
+        for (auto path : paths) {
+            path *= devicePixelRatio;
+            QPolygon polygon(path.toFillPolygon().toPolygon());
+            region->add(polygon);
+        }
+    }
+    blur->setRegion(region);
+    blur->commit();
+    kwayland_surface->commit(Surface::CommitFlag::None);
+}
+
 bool DWaylandShellManager::disableClientDecorations(QWaylandShellSurface *surface)
 {
     Q_UNUSED(surface)
