@@ -1,19 +1,6 @@
-/*
- * Copyright (C) 2017 ~ 2018 Deepin Technology Co., Ltd.
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 - 2022 Uniontech Software Technology Co.,Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
 
 #ifndef UTILITY_H
 #define UTILITY_H
@@ -76,6 +63,10 @@ public:
     static void clearWindowProperty(quint32 WId, xcb_atom_t propAtom);
 
     static void setNoTitlebar(quint32 WId, bool on);
+    static void splitWindowOnScreen(quint32 WId, quint32 type);
+    static void splitWindowOnScreenByType(quint32 WId, quint32 position, quint32 type);
+    static bool supportForSplittingWindow(quint32 WId);
+    static bool supportForSplittingWindowByType(quint32 WId, quint32 screenSplittingType);
 
     struct BlurArea {
         qint32 x;
