@@ -1,19 +1,6 @@
-/*
- * Copyright (C) 2017 ~ 2018 Deepin Technology Co., Ltd.
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 - 2022 Uniontech Software Technology Co.,Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
 
 #include "utility.h"
 #include "qxcbintegration.h"
@@ -232,13 +219,13 @@ void Utility::setFrameExtents(WId wid, const QMargins &margins)
         return;
     }
 
-    int32_t datas[4];
-    datas[0] = int32_t(margins.left());
-    datas[1] = int32_t(margins.right());
-    datas[2] = int32_t(margins.top());
-    datas[3] = int32_t(margins.bottom());
+    int32_t data[4];
+    data[0] = int32_t(margins.left());
+    data[1] = int32_t(margins.right());
+    data[2] = int32_t(margins.top());
+    data[3] = int32_t(margins.bottom());
 
-    xcb_change_property_checked(QX11Info::connection(), XCB_PROP_MODE_REPLACE, xcb_window_t(wid), frameExtents, XCB_ATOM_CARDINAL, 32, 4, datas);
+    xcb_change_property_checked(QX11Info::connection(), XCB_PROP_MODE_REPLACE, xcb_window_t(wid), frameExtents, XCB_ATOM_CARDINAL, 32, 4, data);
 }
 
 static QVector<xcb_rectangle_t> qregion2XcbRectangles(const QRegion &region)
@@ -324,8 +311,6 @@ void Utility::sendMoveResizeMessage(quint32 WId, uint32_t action, QPoint globalP
         globalPos = qApp->primaryScreen()->handle()->cursor()->pos();
     }
 
-    qDebug() << Q_FUNC_INFO << globalPos;
-
     xcb_client_message_event_t xev;
 
     xev.response_type = XCB_CLIENT_MESSAGE;
@@ -474,6 +459,45 @@ void Utility::setNoTitlebar(quint32 WId, bool on)
     }
 }
 
+void Utility::splitWindowOnScreen(quint32 WId, quint32 type)
+{
+    splitWindowOnScreenByType(WId, type, 1);
+}
+
+void Utility::splitWindowOnScreenByType(quint32 WId, quint32 position, quint32 type)
+{
+    xcb_client_message_event_t xev;
+
+    xev.response_type = XCB_CLIENT_MESSAGE;
+    xev.type = internAtom("_DEEPIN_SPLIT_WINDOW", false);
+    xev.window = WId;
+    xev.format = 32;
+    xev.data.data32[0] = position; /* enum class SplitType in kwin-dev, Left=0x1, Right=0x10, Top=0x100, Bottom=0x1000*/
+    xev.data.data32[1] = position == 15 ? 0 : 1; /* 0: not preview 1: preview*/
+    xev.data.data32[2] = type; /* 1:two splitting 2:three splitting 4:four splitting*/
+
+    xcb_send_event(QX11Info::connection(), false, QX11Info::appRootWindow(QX11Info::appScreen()),
+                   SubstructureNotifyMask, (const char *)&xev);
+    xcb_flush(QX11Info::connection());
+}
+
+bool Utility::supportForSplittingWindow(quint32 WId)
+{
+    return Utility::supportForSplittingWindowByType(WId, 1);
+}
+
+// screenSplittingType: 0: can't splitting, 1:two splitting, 2: four splitting(includ three splitting)
+bool Utility::supportForSplittingWindowByType(quint32 WId, quint32 screenSplittingType)
+{
+    auto propAtom = internAtom("_DEEPIN_NET_SUPPORTED");
+    QByteArray data = windowProperty(WId, propAtom, XCB_ATOM_CARDINAL, 4);
+
+    if (const char *cdata = data.constData())
+        return *(reinterpret_cast<const quint8 *>(cdata)) >= screenSplittingType;
+
+    return false;
+}
+
 bool Utility::setEnableBlurWindow(const quint32 WId, bool enable)
 {
     if (!DXcbWMSupport::instance()->hasBlurWindow() || !DXcbWMSupport::instance()->isKwin())
